﻿// -*- C++ -*-
/*!
 * @file  ArUcoOutput2.cpp
 * @brief ${rtcParam.description}
 * @date $Date$
 *
 * $Id$
 */

#include "ArUcoOutput2.h"

// Module specification
// <rtc-template block="module_spec">
static const char* arucooutput2_spec[] =
  {
    "implementation_id", "ArUcoOutput2",
    "type_name",         "ArUcoOutput2",
    "description",       "${rtcParam.description}",
    "version",           "1.0.0",
    "vendor",            "rsdlab",
    "category",          "Category",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    //マーカ情報取得範囲用コンフィギュレーションのデフォルト値(単位:m)
    "conf.default.x_max", "0.05",
    "conf.default.x_min", "-0.05",
    "conf.default.y_max", "1",
    "conf.default.y_min", "-1",

    // Widget
    "conf.__widget__.x_max", "text",
    "conf.__widget__.x_min", "text",
    "conf.__widget__.y_max", "text",
    "conf.__widget__.y_min", "text",
    // Constraints

    "conf.__type__.x_max", "double",
    "conf.__type__.x_min", "double",
    "conf.__type__.y_max", "double",
    "conf.__type__.y_min", "double",

    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
ArUcoOutput2::ArUcoOutput2(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_InPose3DIn("InPose3D", m_InPose3D),
    m_OutPose3DOut("OutPose3D", m_OutPose3D),
    m_OutArUcoIDOut("OutArUcoID", m_OutArUcoID)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
ArUcoOutput2::~ArUcoOutput2()
{
}



RTC::ReturnCode_t ArUcoOutput2::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("InPose3D", m_InPose3DIn);
  
  // Set OutPort buffer
  addOutPort("OutPose3D", m_OutPose3DOut);
  addOutPort("OutArUcoID", m_OutArUcoIDOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  //条件パラメータ用コンフィギュレーション作成
  bindParameter("x_max", m_x_max, "0");
  bindParameter("x_min", m_x_min, "0");
  bindParameter("y_max", m_y_max, "0");
  bindParameter("y_min", m_y_min, "0");
  // </rtc-template>
  
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t ArUcoOutput2::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ArUcoOutput2::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ArUcoOutput2::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t ArUcoOutput2::onActivated(RTC::UniqueId ec_id)
{
  //初期値を設定
  m_OutPose3D.data.position.x = 0;
  m_OutPose3D.data.position.y = 0;
  m_OutPose3D.data.position.z = 0;
  m_OutPose3D.data.orientation.r = 0;
  m_OutPose3D.data.orientation.p = 0;
  m_OutPose3D.data.orientation.y = 0;
  return RTC::RTC_OK;
}


RTC::ReturnCode_t ArUcoOutput2::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t ArUcoOutput2::onExecute(RTC::UniqueId ec_id)
{
  if(m_InPose3DIn.isNew()){
    m_InPose3DIn.read();

    int length = m_InPose3D.ids.length();
    int id = m_InPose3D.ids[0];

    //マーカ情報出力
    if(id !=0 && id != 1023){

      double rot[9];
      double trans[3];
      
      rot[0] = m_InPose3D.rotations[0].R11;
      rot[1] = m_InPose3D.rotations[0].R12;
      rot[2] = m_InPose3D.rotations[0].R13;
      rot[3] = m_InPose3D.rotations[0].R21;
      rot[4] = m_InPose3D.rotations[0].R22;
      rot[5] = m_InPose3D.rotations[0].R23;
      rot[6] = m_InPose3D.rotations[0].R31;
      rot[7] = m_InPose3D.rotations[0].R32;

      rot[8] = m_InPose3D.rotations[0].R33;
      trans[0] = m_InPose3D.translates[0].x;
      trans[1] = m_InPose3D.translates[0].y;
      trans[2] = m_InPose3D.translates[0].z;

      std::cout << "id : " << id << std::endl;
      std::cout << "R11 : " << rot[0] << "R12 : " << rot[1] << "R13 : " << rot[2] << std::endl;
      std::cout << "R21 : " << rot[3] << "R22 : " << rot[4] << "R23 : " << rot[5] << std::endl;
      std::cout << "R31 : " << rot[6] << "R32 : " << rot[7] << "R33 : " << rot[8] << std::endl;
      std::cout << "x : " << trans[0] << "y : " << trans[1] << "z : " << trans[2] << std::endl;

      //アルゴリズム
      if(m_x_min <= trans[0] && trans[0] <= m_x_max && m_y_min <= trans[1] && trans[1] <= m_y_max){
	
      m_OutPose3D.data.position.x = 0;
      m_OutPose3D.data.position.y = 0;
      m_OutPose3D.data.position.z = trans[2];
      m_OutPose3D.data.orientation.r = 0;
      m_OutPose3D.data.orientation.p = 0;
      m_OutPose3D.data.orientation.y = 0;

      m_OutArUcoID.data = id;
      
      m_OutPose3DOut.write();
      m_OutArUcoIDOut.write();

      
      }

      
    }
  }
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t ArUcoOutput2::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ArUcoOutput2::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ArUcoOutput2::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ArUcoOutput2::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ArUcoOutput2::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void ArUcoOutput2Init(RTC::Manager* manager)
  {
    coil::Properties profile(arucooutput2_spec);
    manager->registerFactory(profile,
                             RTC::Create<ArUcoOutput2>,
                             RTC::Delete<ArUcoOutput2>);
  }
  
};


